<!DOCTYPE html>
<html>
<head>
    <title>{% block title %}{% endblock %}</title>

    {# Project's assets #}
    <link href="{{ main_css }}" rel="stylesheet">
    <script src="{{ main_js }}" type="module"></script>
</head>

<body class="vh-100 d-flex align-items-center">
    <main class="card m-auto" style="width: 450px" id="page-login">
        <div class="card-body">
            <img src="{{ brand.logo }}" alt="Logo" id="logo" class="mb-4">
            {% block main %}
            {% endblock %}
        </div>
    </main>
</body>

</html>
