{% extends "base_anon.html" %}

{% block title %}Reset your password{% endblock %}

{% block main %}

<p>
Check your email for a link to reset your password. If it doesn't appear within a few
minutes, check your spam folder.
</p>

<p>
If you don't have an account with this email address, then you won't receive anything.
{% if settings.register %}
Please <a href="{{ url('register') }}">sign up</a>.
{% else %}
Please contact with the site admin to create an account for you.
{% endif %}
</p>

{% if settings.demo %}
<p>
Remember that in the demo site, content and user's database is cleaned up every day.
</p>
{% endif %}

<a href="{{ url('login') }}">Return to sign in</a>

{% endblock %}
