{% extends "base_anon.html" %}

{% block title %}Reset your password{% endblock %}

{% block main %}

<h5 class="card-title float-end">Reset your password</h5>

<div id="result" class="alert alert-danger" style="display: none"></div>

<form action="{{ url('auth/reset-password') }}" method="post"
      onsubmit="submitFormAsJSON(this, '{{ url("login") }}'); return false"
>
    <input type="hidden" name="token" value="{{ token }}">
    <div class="form-floating mb-2">
        <input type="password" name="password" class="form-control" placeholder="" required>
        <label class="form-label">Password</label>
    </div>
    <button type="submit" class="btn btn-primary">Save</button>
    <a class="btn btn-secondary" href="/login" role="button">Cancel</a>
</form>

{% endblock %}
